# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Anaconda Cloud login dialog."""

# yapf: disable

from __future__ import absolute_import, division, print_function

# Standard library imports
import ast
import json
from collections import namedtuple

# Third party imports
from qtpy.QtCore import QRegExp, Qt, QUrl, Signal
from qtpy.QtGui import QDesktopServices, QRegExpValidator, QPixmap
from qtpy.QtWidgets import QApplication, QHBoxLayout, QLabel, QLineEdit, QVBoxLayout, QWidget, QGridLayout

from conda_token.repo_config import CondaTokenError, validate_token, token_set

# Local imports
from anaconda_navigator.utils import get_domain_from_api_url
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config import CONF
from anaconda_navigator.static.fonts import load_fonts
from anaconda_navigator.utils.analytics import GATracker
from anaconda_navigator.utils.styles import load_style_sheet
from anaconda_navigator.widgets import ButtonNormal, ButtonPrimary, FrameBase
from anaconda_navigator.widgets.dialogs import StaticDialogBase, LabelBase, MessageBoxInformation
from anaconda_navigator.widgets.manager.channels import SelectableChannelsListTable
from anaconda_navigator.static.images import (
    ANACONDA_TEAM_EDITION_LOGIN_LOGO, ANACONDA_ENTERPRISE_EDITION_LOGIN_LOGO,
    ANACONDA_ORG_EDITION_LOGIN_LOGO, ANACONDA_COMMERCIAL_EDITION_LOGIN_LOGO)


# yapf: enable

USER_RE = QRegExp(r'^[A-Za-z0-9_\.][A-Za-z0-9_\.\-]+$')
FORGOT_USERNAME_URL = 'account/forgot_username'
FORGOT_PASSWORD_URL = 'account/forgot_password'


TextContainer = namedtuple('TextContainer', (
    'info_frame_text',
    'forgot_links_msg',
    'form_main_text',
    'form_secondary_text',
    'form_input_label_text',
    'form_button_text',
    'message_box_error_text',
    'logo_path'))
TextContainer.__new__.__defaults__ = (None,) * len(TextContainer._fields)

ANACONDA_LOGIN_TEXT_CONTAINER = TextContainer(
    info_frame_text='Log into Anaconda.org to access private channels and packages. If you don’t have an account,'
                    ' click <a href="{}" style="color:#43B049;text-decoration: none">here</a>.',
    forgot_links_msg='Forget your <a href="{username_url}" style="color:#43B049; text-decoration:none">username</a> or '
                     '<a href="{password_url}" style="color:#43B049; text-decoration:none">password</a>?',
    message_box_error_text='The Anaconda.Org API domain is not specified! Please, set in preferences.',
    logo_path=ANACONDA_ORG_EDITION_LOGIN_LOGO
)

ENTERPRISE_LOGIN_TEXT_CONTAINER = TextContainer(
    info_frame_text='Login to configure Conda and Navigator to install packages'
                    ' from your on-premise package repository.',
    message_box_error_text='The Enterprise 4 Repository API domain is not specified! Please, set in preferences.',
    logo_path=ANACONDA_ENTERPRISE_EDITION_LOGIN_LOGO,
)

TEAM_LOGIN_TEXT_CONTAINER = TextContainer(
    logo_path=ANACONDA_TEAM_EDITION_LOGIN_LOGO,
    info_frame_text='Login to configure Conda and Navigator to install packages from your Team Edition instance.',
    message_box_error_text='The Team Edition API domain is not specified! Please, set in preferences.'
)

COMMERCIAL_LOGIN_TEXT_CONTAINER = TextContainer(
    form_main_text='Looks like this is the first time you are logging into Commercial Edition. '
                   'Please set your unique access token.',
    form_secondary_text='You only need to set this token once. You can always change this in your Preferences.',
    form_input_label_text='Enter Commercial Edition Token',
    form_button_text='Set Token',
    logo_path=ANACONDA_COMMERCIAL_EDITION_LOGIN_LOGO,
    info_frame_text='Configure Conda and Navigator to install packages from the open-source distribution optimized '
                    'for commercial use and compliance with our '
                    '<a href="https://www.anaconda.com/terms-of-service" '
                    'style="color:#43B049;text-decoration: none">Terms of Service</a>. '
                    'Subscription required. More details '
                    '<a href="https://www.anaconda.com/products/commercial-edition" '
                    'style="color:#43B049;text-decoration: none">here</a>.'
)

ENTERPRISE_SET_DOMAIN_TEXT_CONTAINER = TextContainer(
    form_main_text='Looks like this is the first time you are logging into Enterprise 4 Repository. '
                   'Please set your Enterprise 4 Repository API domain.',
    form_secondary_text='You only need to set this domain once. You can always change this in your Preferences.',
    form_input_label_text='Enter Enterprise 4 Repository API Domain',
    form_button_text='Set Domain',
    logo_path=ANACONDA_ENTERPRISE_EDITION_LOGIN_LOGO,
    info_frame_text="Login to configure Conda and Navigator to install"
                    " packages from your on-premise package repository."
)

TEAM_SET_DOMAIN_TEXT_CONTAINER = TextContainer(
    form_main_text='Looks like this is the first time you are logging into Team Edition. '
                   'Please set your Team Edition domain.',
    form_secondary_text='You only need to set this domain once. You can always change this in your Preferences.',
    form_input_label_text='Team Edition Domain',
    form_button_text='Set Domain',
    logo_path=ANACONDA_TEAM_EDITION_LOGIN_LOGO,
    info_frame_text="Login to configure Conda and Navigator "
                    "to install packages from your Team Edition instance."
)


class LabelMainLoginTitle(LabelBase):
    """Label used in CSS styling."""


class LabelMainLoginText(LabelBase):
    """Label used in CSS styling."""


class LabelMainLoginSubTitle(LabelBase):
    """Label used in CSS styling."""


class WidgetLoginInfoFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginFormFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginCardsFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginPageContent(FrameBase):
    """Widget used in CSS styling."""


class SecondaryButton(ButtonNormal):
    """Label used in CSS styling."""


class WidgetLoginCard(FrameBase):
    """Widget used in CSS styling."""


class LabelLoginLogo(LabelBase):
    """Label used in CSS styling."""


class BasePage(StaticDialogBase):

    def __init__(self, *args, **kwargs):
        super(BasePage, self).__init__(*args, **kwargs)

    @property
    def username(self):
        """Return the logged username."""
        return self.text_username.text().lower()

    def _get_info_frame(self):
        label_icon = LabelLoginLogo()
        label_icon.setPixmap(QPixmap(self.text_container.logo_path))
        label_icon.setScaledContents(True)  # important on High DPI!
        label_icon.setAlignment(Qt.AlignLeft)

        self.label_information = QLabel(self.text_container.info_frame_text)
        self.label_information.setWordWrap(True)

        self.button_back = SecondaryButton('Back to select screen')

        info_widget = WidgetLoginInfoFrame()
        info_layout = QVBoxLayout()
        info_layout.addWidget(label_icon)
        info_layout.addWidget(self.label_information)
        info_layout.addWidget(self.button_back)
        info_widget.setLayout(info_layout)

        return info_widget

    def _get_forgot_links_widget(self):
        self.forgot_username_url = None
        self.forgot_password_url = None

        forgot_links_widget = QWidget()
        forgot_layout = QHBoxLayout()
        self.forgot_links = QLabel(self.text_container.forgot_links_msg)
        forgot_layout.addWidget(self.forgot_links, 0, Qt.AlignLeft)
        forgot_layout.addStretch(100000000)
        forgot_links_widget.setLayout(forgot_layout)

        return forgot_links_widget

    def open_url(self, url):
        """Open given url in the default browser and log the action."""
        self.tracker.track_event('content', 'click', url)
        QDesktopServices.openUrl(QUrl(url))

    def update_style_sheet(self, style_sheet=None):
        """Update custom css style sheet."""
        if style_sheet is None:
            style_sheet = load_style_sheet()
        self.setStyleSheet(style_sheet)

    def back_to_select_screen(self):
        self.reject()
        self.main_dialog.show()


class BaseLoginPage(BasePage):
    def __init__(self, anaconda_api, main_dialog, text_container):
        super(BaseLoginPage, self).__init__(parent=main_dialog.parent)
        self.text_container = text_container

        self.token = None
        self.error = None
        self.config = CONF
        self.anaconda_api = anaconda_api
        self.main_dialog = main_dialog
        self.tracker = GATracker()

        forgot_links_widget = self._get_forgot_links_widget() if self.text_container.forgot_links_msg else None
        login_form_widget = self._get_form_frame(forgot_links_widget)
        info_widget = self._get_info_frame()

        main_layout = QVBoxLayout()
        body_page_widget = WidgetLoginPageContent()
        body_layout = QHBoxLayout()

        title = LabelMainLoginTitle("Sign in to access your repository")
        title.setWordWrap(True)
        main_layout.addWidget(title)
        body_layout.addWidget(info_widget)
        body_layout.addWidget(login_form_widget)
        body_page_widget.setLayout(body_layout)
        main_layout.addWidget(body_page_widget)
        self.setLayout(main_layout)

        self.text_username.textEdited.connect(self.check_text)
        self.text_password.textEdited.connect(self.check_text)
        self.button_login.clicked.connect(self.login)
        self.button_back.clicked.connect(self.back_to_select_screen)

        self.check_text()
        self.update_style_sheet()
        self.text_username.setFocus()

    def check_text(self):
        """Check that `username` and `password` are not empty strings.

        If not empty and disable/enable buttons accordingly.
        """
        username = self.text_username.text()
        password = self.text_password.text()

        if not all((len(username), len(password))):
            self.button_login.setDisabled(True)
        else:
            self.button_login.setDisabled(False)

    def login(self):
        api_url = self.config.get('main', self.api_url_config_option)
        if not api_url:
            msg_box = MessageBoxInformation(
                title='Login Error',
                text=self.text_container.domain_not_found_msg,
            )
            msg_box.exec_()

            self.button_login.setDisabled(False)
            self.check_text()
            QApplication.restoreOverrideCursor()
            return

        username_text = self.text_username.text().lower()
        self.button_login.setEnabled(False)
        self.label_message.setText('')
        self.text_username.setText(username_text)

        QApplication.setOverrideCursor(Qt.WaitCursor)

        # Reload the client to the other one, if needed.
        self.config.set('main', 'logged_api_url', api_url)
        self.anaconda_api.client_reload()

        worker = self.anaconda_api.login(username_text, self.text_password.text())
        worker.sig_finished.connect(self._finished)

    def _finished(self, worker, output, error):
        """
        Callback for the login procedure after worker has finished.

        If success, sets the token, 'username' attribute to parent widget
        and sends the accept signal.

        Otherwise, outputs error messages.
        """
        token = output
        username = self.text_username.text().lower()

        if token:
            self.token = token
            self.sig_authentication_succeeded.emit()
            self.main_dialog.username = username
            self.accept()
            self.main_dialog.accept()

        elif error:
            bold_username = '<b>{0}</b>'.format(username)

            # The error might come in (error_message, http_error) format
            try:
                error_message = ast.literal_eval(str(error))[0]
            except Exception:  # pragma: no cover
                error_message = str(error)

            error_message = error_message.lower().capitalize()
            error_message = error_message.split(', ')[0]
            error_text = '<i>{0}</i>'.format(error_message)
            error_text = error_text.replace(username, bold_username)
            self.label_message.setText(error_text)
            self.label_message.setVisible(True)

            if error_message:
                domain = self.anaconda_api.client_domain()
                label = '{0}/{1}: {2}'.format(domain, username, error_message.lower())
                self.tracker.track_event('authenticate', 'login failed', label=label)
                self.text_password.setFocus()
                self.text_password.selectAll()
            self.sig_authentication_failed.emit()

        self.button_login.setDisabled(False)
        self.check_text()
        QApplication.restoreOverrideCursor()

    def _get_form_frame(self, forgot_links_widget=None):
        self.label_username = QLabel('Username:')
        self.label_password = QLabel('Password:')
        self.text_username = QLineEdit()
        self.text_password = QLineEdit()
        self.label_message = LabelMainLoginText('')
        self.label_message.setWordWrap(True)

        self.text_username.setValidator(QRegExpValidator(USER_RE))
        self.text_password.setEchoMode(QLineEdit.Password)
        self.label_message.setVisible(False)

        self.button_login = ButtonPrimary('Sign In')
        self.button_login.setDefault(True)

        login_form_widget = WidgetLoginFormFrame()
        login_form_layout = QVBoxLayout()
        for widget in (self.label_username, self.text_username,
                       self.label_password, self.text_password, self.label_message):
            login_form_layout.addWidget(widget)

        if forgot_links_widget:
            login_form_layout.addWidget(forgot_links_widget)

        login_form_layout.addWidget(self.button_login, 0, Qt.AlignHCenter)
        login_form_widget.setLayout(login_form_layout)

        return login_form_widget


class BaseSettingPage(BasePage):
    def __init__(self, anaconda_api, main_dialog, text_container):

        super(BaseSettingPage, self).__init__(parent=main_dialog.parent)

        self.config = CONF
        self.text_container = text_container
        self.anaconda_api = anaconda_api
        self.main_dialog = main_dialog
        self.tracker = GATracker()

        info_widget = self._get_info_frame()
        login_form_widget = self._get_form_frame()

        title = LabelMainLoginTitle("Sign in to access your repository")
        title.setWordWrap(True)

        body_layout = QHBoxLayout()
        body_layout.addWidget(info_widget)
        body_layout.addWidget(login_form_widget)

        body_page_widget = WidgetLoginPageContent()
        body_page_widget.setLayout(body_layout)

        main_layout = QVBoxLayout()
        main_layout.addWidget(title)
        main_layout.addWidget(body_page_widget)
        self.setLayout(main_layout)

        self.button_back.clicked.connect(self.back_to_select_screen)
        self.label_information.linkActivated.connect(lambda activated_link: self.open_url(activated_link))

        self.update_style_sheet()
        self.input_line.setFocus()

    def _get_form_frame(self):
        self.label_text = LabelMainLoginText(self.text_container.form_main_text)
        self.label_note = LabelMainLoginSubTitle(self.text_container.form_secondary_text)
        self.input_label = QLabel(self.text_container.form_input_label_text)
        self.label_text.setWordWrap(True)
        self.label_note.setWordWrap(True)

        self.input_line = QLineEdit()

        self.label_message = LabelMainLoginText('')
        self.label_message.setWordWrap(True)
        self.label_message.setVisible(False)

        self.button_apply = ButtonPrimary(self.text_container.form_button_text)
        self.button_apply.setEnabled(True)
        self.button_apply.setDefault(True)

        login_form_widget = WidgetLoginFormFrame()
        login_form_layout = QVBoxLayout()
        for widget in (self.label_text, self.label_note, self.input_label, self.input_line, self.label_message):
            login_form_layout.addWidget(widget)
        login_form_layout.addWidget(self.button_apply, 0, Qt.AlignHCenter)
        login_form_widget.setLayout(login_form_layout)

        return login_form_widget

    def set_domain(self):
        self.input_line.setText(self.input_line.text().lower())
        self.label_message.setText('')

        if self.check_text():
            self.config.set('main', self.api_url_config_option, self.input_line.text().strip('/'))
            self.anaconda_api.client_reload()
            self.accept()
            self.main_dialog.sig_update_card_handler.emit(self.api_url_config_option)
            login_page = self.next_dialog_factory(AnacondaAPI(), self.main_dialog)
            self.main_dialog.sig_login_type_clicked.emit(login_page)

        QApplication.restoreOverrideCursor()

    def check_text(self, text=None):
        text = text or self.input_line.text()
        valid, error = self.is_valid_api(text.lower())

        if not valid:
            self.label_message.setText(error)
            self.label_message.setVisible(bool(self.input_line.text()))
            return False

        self.button_apply.setEnabled(True)
        self.label_message.setVisible(False)

        return True

    def is_valid_api(self, url, verify=True, allow_blank=False):
        """Check if a given URL is a valid anaconda api endpoint."""

        valid = self.anaconda_api.download_is_valid_api_url(
            url, non_blocking=False, verify=verify, allow_blank=allow_blank
        )
        error = ''
        if not valid:
            error = 'Invalid API url. Check the url is valid and corresponds to the api endpoint.'
        return valid, error


class CommercialEditionLoginPage(BaseSettingPage):

    def __init__(self, anaconda_api, main_dialog):

        super(CommercialEditionLoginPage, self).__init__(anaconda_api, main_dialog, COMMERCIAL_LOGIN_TEXT_CONTAINER)

        self.button_apply.clicked.connect(self.set_token)
        self.input_line.setEchoMode(QLineEdit.Password)

    def set_token(self):
        ce_token = self.input_line.text()
        self.label_message.setText('')

        if self.check_text():
            token_set(ce_token)
            commercial_edition_url = self.config.get('main', 'commercial_edition_url')
            self.config.set('main', 'logged_api_url', commercial_edition_url)
            self.anaconda_api.client_reload()
            self.accept()
            self.main_dialog.accept()

        QApplication.restoreOverrideCursor()

    def check_text(self):
        try:
            validate_token(self.input_line.text())
        except CondaTokenError as e:
            self.label_message.setText(str(e))
            self.label_message.setVisible(bool(self.input_line.text()))
            return False

        self.button_apply.setEnabled(True)
        self.label_message.setVisible(False)

        return True


class TeamEditionSetDomainPage(BaseSettingPage):

    def __init__(self, anaconda_api, main_dialog):
        super(TeamEditionSetDomainPage, self).__init__(anaconda_api, main_dialog, TEAM_SET_DOMAIN_TEXT_CONTAINER)
        self.api_url_config_option = 'team_edition_api_url'
        self.next_dialog_factory = TeamEditionLoginPage
        self.input_line.setPlaceholderText('http(s)://example.com')
        self.button_apply.clicked.connect(self.set_domain)

    def check_text(self, text=None):
        text = text or self.input_line.text()
        domain = get_domain_from_api_url(text)
        self.input_line.setText(domain)
        return super(TeamEditionSetDomainPage, self).check_text(text='{}/api/system'.format(domain))


class TeamEditionLoginPage(BaseLoginPage):
    sig_authentication_succeeded = Signal()
    sig_authentication_failed = Signal()

    def __init__(self, anaconda_api, main_dialog):
        super(TeamEditionLoginPage, self).__init__(anaconda_api, main_dialog, TEAM_LOGIN_TEXT_CONTAINER)
        self.api_url_config_option = 'team_edition_api_url'

    def _finished(self, worker, output, error):
        """
        Callback for the login procedure after worker has finished.

        If success, sets the token, 'username' attribute to parent widget
        and sends the accept signal.

        Otherwise, outputs error messages.
        """
        try:
            response = json.loads(output)

            if 'token' in response and 'refresh_token' in response:
                self.main_dialog.username = self.username
                self.token = output
                self.accept()
                te_channels = TeamEditionAddChannelsPage(self.anaconda_api, self.main_dialog, self)
                te_channels.open()

            else:
                if 'message' in response and response['message'] == 'Unauthorized':
                    self.label_message.setText('<i>Invalid Credentials!</i>')
                elif 'message' in response:
                    self.label_message.setText('<i>{}</i>'.format(response['message']))

                self.label_message.setVisible(True)
                self._track_error(output)

        except (json.decoder.JSONDecodeError, TypeError):
            self.label_message.setText('<i>Unhandled error happened!</i>')
            self.label_message.setVisible(True)
            self._track_error(error)

        self.button_login.setDisabled(False)
        self.check_text()
        QApplication.restoreOverrideCursor()

    def _track_error(self, error):
        domain = self.anaconda_api.client_domain()
        label = '{0}/{1}: {2}'.format(domain, self.username, str(error).lower())
        self.tracker.track_event('authenticate', 'login failed', label=label)
        self.text_password.setFocus()
        self.text_password.selectAll()


class TeamEditionAddChannelsPage(BasePage):
    def __init__(
            self,
            anaconda_api,
            main_dialog=None,
            parent=None,
            msg='Select default channels to be used',
            btn_add_msg='Add Channels'
    ):
        super(TeamEditionAddChannelsPage, self).__init__(main_dialog.parent if main_dialog else parent)

        self.config = CONF
        self.anaconda_api = anaconda_api
        self.main_dialog = main_dialog
        self.parent = parent
        self.tracker = GATracker()

        QApplication.restoreOverrideCursor()
        pixmap = QPixmap(ANACONDA_TEAM_EDITION_LOGIN_LOGO)
        pixmap = pixmap.scaledToWidth(200, Qt.SmoothTransformation)
        label_icon = LabelLoginLogo()
        label_icon.setPixmap(pixmap)
        label_icon.setAlignment(Qt.AlignLeft)
        self.label_information = QLabel(msg)

        rc_data = self.anaconda_api._conda_api.load_rc()

        api_channels_data = self.anaconda_api.get_channels()
        channels = rc_data.get('channels', [])
        default_channels = rc_data.get('default_channels', [])

        self.channels_table = SelectableChannelsListTable(
            self, table_data=api_channels_data, channels=channels, default_channels=default_channels
        )
        self.channels_table.setMaximumWidth(650)

        self.button_skip = SecondaryButton('Skip')
        self.button_add = ButtonPrimary(btn_add_msg)

        buttons_layout = QHBoxLayout()
        buttons_layout.addWidget(self.button_skip)
        buttons_layout.addWidget(self.button_add)

        self.main_layout = QVBoxLayout()
        self.main_layout.addWidget(label_icon, Qt.AlignLeft)
        self.main_layout.addWidget(self.label_information, Qt.AlignRight)
        self.main_layout.addWidget(self.channels_table, Qt.AlignCenter)
        self.main_layout.addLayout(buttons_layout)

        self.setLayout(self.main_layout)

        self.button_skip.clicked.connect(self.skip)
        self.button_add.clicked.connect(self.add_channels)
        self.button_close_dialog.clicked.connect(self.close_window)

    def skip(self):
        self.anaconda_api.create_login_data()
        self.close_window()

    def add_channels(self):
        default_channels, channels = self.channels_table.get_selected_channels()

        if not default_channels:
            self.label_information.setText(
                'At least one channel should be added to <b>default_channels</b>! Please add...'
            )
        else:
            self.anaconda_api.create_login_data(default_channels, channels)
            self.close_window()

    def close_window(self):
        try:
            self.parent.accept()
            self.main_dialog.accept()
        except AttributeError:
            self.parent.sig_logged_in.emit()

        self.accept()


class EnterpriseRepoSetDomainPage(BaseSettingPage):
    def __init__(self, anaconda_api, main_dialog):
        """Login dialog."""

        super(EnterpriseRepoSetDomainPage, self).__init__(
            anaconda_api, main_dialog, ENTERPRISE_SET_DOMAIN_TEXT_CONTAINER)
        self.api_url_config_option = 'enterprise_4_repo_api_url'
        self.next_dialog_factory = EnterpriseRepoLoginPage
        self.input_line.setPlaceholderText('http(s)://example.com')
        self.button_apply.clicked.connect(self.set_domain)


class EnterpriseRepoLoginPage(BaseLoginPage):

    sig_authentication_succeeded = Signal()
    sig_authentication_failed = Signal()

    def __init__(self, anaconda_api, main_dialog):
        """Login dialog."""

        super(EnterpriseRepoLoginPage, self).__init__(anaconda_api, main_dialog, ENTERPRISE_LOGIN_TEXT_CONTAINER)
        self.api_url_config_option = 'enterprise_4_repo_api_url'


class AnacondaLoginPage(BaseLoginPage):
    sig_authentication_succeeded = Signal()
    sig_authentication_failed = Signal()

    def __init__(self, anaconda_api, main_dialog):
        """Login dialog."""

        super(AnacondaLoginPage, self).__init__(anaconda_api, main_dialog, ANACONDA_LOGIN_TEXT_CONTAINER)
        self.api_url_config_option = 'anaconda_api_url'
        self.update_links()

    def update_links(self):
        """Update links."""

        anaconda_api_url = self.config.get('main', 'anaconda_api_url', self.anaconda_api.client_get_api_url())
        if not anaconda_api_url:
            return

        base_url = anaconda_api_url.lower().replace('//api.', '//')

        parts = base_url.lower().split('/')
        base_url = '/'.join(parts[:-1]) if parts[-1] == 'api' else base_url

        forgot_links_updated_text = self.forgot_links.text().format(
            username_url=base_url + '/' + FORGOT_USERNAME_URL,
            password_url=base_url + '/' + FORGOT_PASSWORD_URL
        )
        info_updated_text = self.label_information.text().format(base_url)
        self.label_information.setText(info_updated_text)
        self.forgot_links.setText(forgot_links_updated_text)

        self.label_information.linkActivated.connect(lambda activated_link: self.open_url(activated_link))
        self.forgot_links.linkActivated.connect(lambda activated_link: self.open_url(activated_link))


class LoginCard(QWidget):
    def __init__(
        self,
        description=None,
        image_path=None,
        login_page=None,
        login_signal=None,
    ):
        """Item with custom widget for the applications list."""
        super(LoginCard, self).__init__()
        self.login_page = login_page

        self.api = AnacondaAPI()
        self.description = description
        self.image_path = image_path
        self.style_sheet = None

        self.button_login = SecondaryButton("SIGN IN")
        self.label_description = QLabel(self.description)

        self.label_icon = LabelLoginLogo()
        self.pixmap = QPixmap(self.image_path)

        self.label_icon.setPixmap(self.pixmap)
        self.label_icon.setScaledContents(True)  # important on High DPI!
        self.label_icon.setMaximumWidth(200)
        self.label_icon.setMaximumHeight(52)
        self.label_icon.setAlignment(Qt.AlignLeft)

        self.label_description.setWordWrap(True)
        self.label_description.setAlignment(Qt.AlignLeft)

        body_widget = WidgetLoginCard()
        layout_main = QVBoxLayout()
        layout_body = QVBoxLayout()
        layout_body.addWidget(self.label_icon, 0, Qt.AlignLeft)
        layout_body.addWidget(self.label_description, 0, Qt.AlignLeft)
        layout_body.addWidget(self.button_login, 0, Qt.AlignRight)

        body_widget.setLayout(layout_body)

        layout_main.addWidget(body_widget)
        self.setLayout(layout_main)

        self.button_login.clicked.connect(lambda: login_signal.emit(self.login_page))


class MainLoginDialog(StaticDialogBase):
    sig_login_type_clicked = Signal(object)
    sig_update_card_handler = Signal(str)

    def __init__(self, parent=None):
        """Login dialog."""

        super(MainLoginDialog, self).__init__(parent)
        self.parent = parent

        self.username = None

        title = LabelMainLoginTitle("Sign in to access your repository")
        title.setWordWrap(True)

        description = LabelMainLoginText(
            "Connect to your repository to enable powerful collaboration"
            " and package management for open-source and private projects."
        )
        description.setWordWrap(True)

        subtitle = LabelMainLoginSubTitle("Connect to your Repository")
        subtitle.setWordWrap(True)

        grid_widget = WidgetLoginCardsFrame()
        grid = QGridLayout()

        self.anaconda_client_login_card = LoginCard(
            description="Log into Anaconda.org to access private channels and packages.",
            image_path=ANACONDA_ORG_EDITION_LOGIN_LOGO,
            login_page=AnacondaLoginPage(AnacondaAPI(), self),
            login_signal=self.sig_login_type_clicked
        )
        self.team_edition_login_card = LoginCard(
            description="Login to configure Conda and Navigator to "
            "install packages from your on-premise package repository.",
            image_path=ANACONDA_TEAM_EDITION_LOGIN_LOGO,
            login_page=self._get_team_edition_page(),
            login_signal=self.sig_login_type_clicked
        )
        self.commercial_edition_login_card = LoginCard(
            description="Configure Conda and Navigator to install packages from the open-source distribution optimized "
            "for commercial use and compliance with our Terms of Service.",
            image_path=ANACONDA_COMMERCIAL_EDITION_LOGIN_LOGO,
            login_page=CommercialEditionLoginPage(AnacondaAPI(), self),
            login_signal=self.sig_login_type_clicked
        )
        self.enterprise_edition_login_card = LoginCard(
            description="Login to configure Conda and Navigator to "
            "install packages from your on-premise package repository.",
            image_path=ANACONDA_ENTERPRISE_EDITION_LOGIN_LOGO,
            login_page=self._get_enterprise_page(),
            login_signal=self.sig_login_type_clicked
        )

        grid.addWidget(self.anaconda_client_login_card, 1, 1)
        grid.addWidget(self.team_edition_login_card, 1, 2)
        grid.addWidget(self.commercial_edition_login_card, 2, 1)
        grid.addWidget(self.enterprise_edition_login_card, 2, 2)
        grid_widget.setLayout(grid)

        main_layout = QVBoxLayout()
        main_layout.addWidget(title)
        main_layout.addWidget(description)
        main_layout.addWidget(subtitle)
        main_layout.addWidget(grid_widget)
        self.setLayout(main_layout)

        self.sig_login_type_clicked.connect(self.login_page_selected)
        self.sig_update_card_handler.connect(self.update_card_handler)

    def _get_enterprise_page(self):
        enterprise_api_url = CONF.get('main', 'enterprise_4_repo_api_url')
        return EnterpriseRepoLoginPage(AnacondaAPI(), self) if enterprise_api_url \
            else EnterpriseRepoSetDomainPage(AnacondaAPI(), self)

    def _get_team_edition_page(self):
        team_edition_api_url = CONF.get('main', 'team_edition_api_url')
        if team_edition_api_url:
            return TeamEditionLoginPage(AnacondaAPI(), self)

        return TeamEditionSetDomainPage(AnacondaAPI(), self)

    def login_page_selected(self, login_page):
        if not self.isHidden():
            self.hide()
        login_page.open()

    def update_card_handler(self, domain_url_key):
        if domain_url_key == 'enterprise_4_repo_api_url':
            self.enterprise_edition_login_card.login_page = EnterpriseRepoLoginPage(AnacondaAPI(), self)
        elif domain_url_key == 'team_edition_api_url':
            self.team_edition_login_card.login_page = TeamEditionLoginPage(AnacondaAPI(), self)


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local test."""
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication(test_time=3)
    load_fonts(app)
    widget = MainLoginDialog()
    widget.show()
    app.exec_()


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Conda channel selector dialog."""

# yapf: disable

from __future__ import (absolute_import, division, print_function,
                        with_statement)

# Standard library imports
import os
import sys

# Third party imports
from qtpy.QtCore import QEvent, QRegExp, QSize, Qt, Signal
from qtpy.QtGui import QKeySequence, QRegExpValidator
from qtpy.QtWidgets import (QAbstractItemView, QAction, QApplication, QFrame,
                            QHBoxLayout, QListWidget, QListWidgetItem, QMenu,
                            QProgressBar, QSizePolicy, QVBoxLayout)

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config.main import CONF
from anaconda_navigator.utils.misc import path_is_writable
from anaconda_navigator.utils.styles import SASS_VARIABLES, load_style_sheet
from anaconda_navigator.widgets import (ButtonDanger, ButtonNormal,
                                        ButtonPrimary, FrameBase, LabelBase,
                                        LineEditBase, SpacerHorizontal,
                                        SpacerVertical)
from anaconda_navigator.widgets.dialogs import DialogBase


# yapf: enable


# --- Widgets used in CSS styling
# -----------------------------------------------------------------------------
class LabelConfigLocation(LabelBase):
    """Label displaying the configuration location."""


class LabelChannelInfo(LabelBase):
    """Label displaying channel information."""


class FrameChannels(FrameBase):
    """Frame used in CSS styling."""


class ButtonRemoveChannel(ButtonDanger):
    """Button that emits signal on focus."""

    sig_focused = Signal()

    def focusInEvent(self, event):
        """Override Qt method."""
        super(ButtonRemoveChannel, self).focusInEvent(event)
        self.sig_focused.emit()


# --- Heper widgets
# -----------------------------------------------------------------------------
class WorkerMock:
    """Worker mock to handle `defaults` channel special case."""
    def __init__(self, item=None, url=None, repodata_url=None):
        """Worker mock to handle `defaults` channel special case."""
        self.item = item
        self.url = url
        self.repodata_url = url


class LineEditChannel(LineEditBase):
    """
    Custom line edit that uses different validators for text and url.

    More info:
    http://conda.pydata.org/docs/config.html#channel-locations-channels

    Valid entries:
    - defaults  <- Special case
    - <some-channel-name>
    - https://conda.anaconda.org/<channel>/<package>
    - https://conda.anaconda.org/t/<token>/<package>
    - http://<some.custom.url>/<channel>
    - https://<some.custom.url>/<channel>
    - file:///<some-local-directory>
    """

    VALID_RE = QRegExp('^[A-Za-z][A-Za-z0-9/_-]+$|' '^https?://.*|' '^file:///.*')

    sig_return_pressed = Signal()
    sig_escape_pressed = Signal()
    sig_copied = Signal()

    def __init__(self, *args, **kwargs):
        """Custom line edit that uses different validators for text and url."""
        super(LineEditChannel, self).__init__(*args, **kwargs)
        self._validator = QRegExpValidator(self.VALID_RE)
        self.menu = QMenu(parent=self)
        self.setValidator(self._validator)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

    def event(self, event):
        """Override Qt method."""
        if (event.type() == QEvent.MouseButtonPress and event.buttons() & Qt.RightButton and not self.isEnabled()):
            self.show_menu(event.pos())
            return True
        else:
            return super(LineEditChannel, self).event(event)

    def keyPressEvent(self, event):
        """Override Qt method."""
        key = event.key()

        # Display a copy menu in case the widget is disabled.
        if event.matches(QKeySequence.Paste):
            clipboard = QApplication.clipboard()
            text = clipboard.text()
            if self.VALID_RE.exactMatch(text):
                self.setText(text)
                return
        else:
            if key in [Qt.Key_Return, Qt.Key_Enter]:
                self.sig_return_pressed.emit()
            elif key in [Qt.Key_Escape]:
                self.sig_escape_pressed.emit()
        super(LineEditChannel, self).keyPressEvent(event)

    def show_menu(self, pos):
        """Show copy menu for channel item."""
        self.menu.clear()
        copy = QAction("&Copy", self.menu)
        copy.triggered.connect(self.copy_text)
        self.menu.addAction(copy)
        self.menu.setEnabled(True)
        self.menu.exec_(self.mapToGlobal(pos))

    def copy_text(self):
        """Copy channel text to clipboard."""
        clipboard = QApplication.clipboard()
        clipboard.setText(self.text())
        self.sig_copied.emit()


# --- Main list widgets
# -----------------------------------------------------------------------------
class ListWidgetItemChannel(QListWidgetItem):
    """Conda channels list widget item used in CSS styling."""
    def __init__(self, channel=None, location=None):
        """Conda channels list widget item used in CSS styling."""
        super(ListWidgetItemChannel, self).__init__()
        self.channel = channel if channel else ''
        self.location = location if location else ''
        self.new_channel_help_text = "Enter a channel name, " \
                                     "file location, or url."
        location_label = "Path to configuration file: {}".format(location)

        # Widgets
        self.widget = FrameChannels()
        self.new_channel_help = LabelConfigLocation(self.new_channel_help_text)
        self.new_channel_help.setVisible(False)
        self.label_location = LabelConfigLocation(location_label)
        self.label_location.setVisible(False)
        self.text_channel = LineEditChannel()
        self.label_info = LabelChannelInfo()
        self.button_remove = ButtonRemoveChannel()

        # Widgets setup
        self.button_remove.setVisible(path_is_writable(location))
        self.text_channel.setText(channel)

        # Layouts
        layout_name = QVBoxLayout()
        layout_name.addWidget(self.text_channel)
        layout_name.addWidget(self.new_channel_help)
        layout_name.addWidget(self.label_location)
        self.label_location.setToolTip(location_label)

        layout_frame = QHBoxLayout()
        layout_frame.addLayout(layout_name)
        layout_frame.addStretch()
        layout_frame.addWidget(self.label_info)
        layout_frame.addWidget(self.button_remove)
        self.widget.setLayout(layout_frame)
        self.setSizeHint(self.widget_size())

    def set_editable(self, value):
        """Set the editable status of the channel textbox."""
        self.text_channel.setEnabled(value)
        self.text_channel.setFocus()

    @staticmethod
    def widget_size():
        """Return the size defined in the SASS file."""
        return QSize(SASS_VARIABLES.WIDGET_CHANNEL_TOTAL_WIDTH, SASS_VARIABLES.WIDGET_CHANNEL_TOTAL_HEIGHT)


class ListWidgetChannels(QListWidget):
    """Conda channels list widget."""

    sig_channel_added = Signal(object)
    sig_channel_removed = Signal(object)
    sig_channel_status = Signal(bool)
    sig_channel_checked = Signal()
    sig_status_updated = Signal(object, object, object, object)
    sig_focus_fixed = Signal()

    def __init__(
        self,
        parent=None,
        api=None,
        main_url='https://anaconda.org',
        api_url='https://api.anaconda.org',
        conda_url='https://conda.anaconda.org'
    ):
        """Conda channels list widget."""
        super(ListWidgetChannels, self).__init__(parent)

        # Variables
        self._items = []
        self.api = api
        self.api_url = api_url
        self.main_url = main_url
        self.conda_url = conda_url
        self.style_sheet = None
        self.repeat_error = False

        # Widget setup
        self.setObjectName('ListWidgetChannels')
        self.setResizeMode(QListWidget.Adjust)
        self.setMovement(QListWidget.Static)
        self.setFrameStyle(QListWidget.Plain)
        self.setSelectionMode(QAbstractItemView.NoSelection)
        self.setViewMode(QListWidget.ListMode)
        self.setFocusPolicy(Qt.NoFocus)
        self.setUniformItemSizes(True)

    def addItem(self, item):
        """
        Override Qt method.

        Add a content item to the list.
        """
        super(ListWidgetChannels, self).addItem(item)
        self.setItemWidget(item, item.widget)

        item.text_channel.sig_return_pressed.connect(lambda: self.validate_channel(item))
        item.text_channel.sig_escape_pressed.connect(lambda: self.escape_pressed(item))
        item.button_remove.clicked.connect(lambda: self.remove_channel(item))
        item.button_remove.sig_focused.connect(lambda v=None: self.fix_focus(item))

        # Adding an empty channel triggers edit mode
        if not item.channel.strip().lower():
            self.is_editing = True
            self.scrollToBottom()
            item.set_editable(True)
            item.text_channel.textEdited.connect(self.check_repeat)
            item.button_remove.setVisible(False)
        else:
            self.is_editing = False
            self._items.append(item)

        item.text_channel.setToolTip(item.channel)
        item.label_info.setDisabled(True)

    def escape_pressed(self, item):
        """Handle cancellation of ongoing new channel editing."""
        self.takeItem(self.count() - 1)

    def fix_focus(self, item):
        """Set the current row based on focus of child widgets."""
        row = self._items.index(item)
        self.setCurrentRow(row)
        self.sig_focus_fixed.emit()

    def update_style_sheet(self, style_sheet=None):
        """Update custom CSS style sheet."""
        if style_sheet is None:
            self.style_sheet = load_style_sheet()
        else:
            self.style_sheet = style_sheet

        for item in self._items:
            try:
                item_widget = self.itemWidget(item)
                item_widget.setStyleSheet(self.style_sheet)
                item.text_channel.menu.setStyleSheet(self.style_sheet)
                item.setSizeHint(item.widget_size())
            except Exception:  # pragma: no cover
                pass
                # This error is just in case the C++ object has been
                # deleted and it is not crucial to log.
        self.update()
        self.repaint()

    def remove_channel(self, item):
        """Remove the selected channel."""
        if item in self._items:
            index = self._items.index(item)
            self.takeItem(index)
            self._items.remove(item)
            self.sig_channel_removed.emit(item.channel)

    def _channel_url_validated(self, worker, valid, error):
        """Callback for channel url validation."""
        self.setDisabled(False)
        item = worker.item
        if valid:
            item.set_editable(False)
            item.channel = worker.channel
            item.text_channel.setText(worker.channel)
            item.text_channel.setToolTip(worker.channel)
            item.new_channel_help.setVisible(False)
            item.label_location.setVisible(False)
            item.button_remove.setVisible(True)
            self.show_tool_tip(item)
            self._items.append(item)
            self.sig_channel_added.emit(worker.channel)
            self.sig_channel_status.emit(True)
            self.update_style_sheet(self.style_sheet)
            self.sig_status_updated.emit('', '', None, None)
        else:
            text = ('<b>{0}</b> is not a valid conda ' 'channel.'.format(worker.url))
            self.sig_status_updated.emit('Invalid channel', '', None, None)
            item.set_editable(True)
            self.show_tool_tip(item, text=text)
            self.sig_channel_status.emit(False)

        self.sig_channel_checked.emit()

    @staticmethod
    def show_tool_tip(item, text=''):
        """Set the tooltip in case of errors."""
        widget = item.label_info
        widget.setToolTip(text)
        widget.setEnabled(bool(text))

    def check_repeat(self, text):
        """Check that given channel (text) and source is not in list."""
        compare = []
        for source, channels in self.sources.items():
            for channel in channels:
                lower_channel = channel.lower()
                compare.append((source, lower_channel))

        text = text.lower().strip()
        current_compare = (self.api._conda_api.user_rc_path, text)
        self.repeat_error = current_compare in compare

    def validate_channel(self, item):
        """Validate entered channel with current api url."""
        channel = item.text_channel.text().strip()
        self.sig_status_updated.emit('Validating channel...', '', 0, 0)
        item.channel = channel

        if self.repeat_error:
            # Channel is already in list
            self.show_tool_tip(item, text='Channel <b>{0}</b> is in list ' 'already.'.format(channel))
            worker = WorkerMock(item=item, url='', repodata_url='')
            worker.channel = channel
            self._channel_url_validated(worker, False, None)
        elif channel:
            # Try to validate
            if channel == 'defaults':
                url = []  # no need to check this special channel
                worker = WorkerMock(item=item, url=channel, repodata_url=channel)
                worker.channel = channel
                return self._channel_url_validated(worker, True, None)
            elif channel.startswith(('https://', 'http://')):
                if channel.startswith(self.main_url):
                    # User entered https://anaconda.org/<channel> or
                    # User entered https://anaconda.org/t/<token>/<package>
                    url = channel.replace(self.main_url, self.conda_url)
                    channel = url
                else:
                    url = channel
            elif channel.startswith('file:///'):
                # Its a local folder, only check that the folder exists
                url = []  # no need to check this special channel
                worker = WorkerMock(item=item, url=channel, repodata_url=channel)
                worker.channel = channel
                if os.name == 'nt':  # pragma: no cover unix
                    path = channel.replace('file:///', '')
                else:  # pragma: no cover windows
                    path = channel.replace('file://', '')
                valid = os.path.isdir(path)
                return self._channel_url_validated(worker, valid, None)
            else:
                logged_api_url = CONF.get('main', 'logged_api_url')
                token = CONF.get('main', 'team_edition_token')

                if logged_api_url == CONF.get('main', 'team_edition_api_url'):
                    url = "{0}/t/{1}/{2}".format(self.conda_url, token, channel)
                else:
                    url = '{0}/{1}'.format(self.conda_url, channel)

            url = url[:-1] if url[-1] == '/' else url
            repodata_url = "{0}/noarch/{1}".format(url, 'repodata.json')

            worker = self.api.download_is_valid_url(repodata_url)
            worker.sig_finished.connect(self._channel_url_validated)
            worker.item = item
            worker.url = url
            worker.channel = channel
            worker.repodata_url = repodata_url
            self.setDisabled(True)
            self.show_tool_tip(item)
        else:
            # Inform user channel is empty!
            self.show_tool_tip(item, text='Channel cannot be empty.')
            self.sig_status_updated.emit('Channel cannot be empty', '', None, None)
            worker = WorkerMock(item=item, url='', repodata_url='')
            worker.channel = channel
            self._channel_url_validated(worker, False, None)

    @property
    def sources(self):
        """Return the channels."""
        sources = [item.location for item in self._items]
        config_sources = {}
        for source in sources:
            if source not in config_sources:
                config_sources[source] = []

        for item in self._items:
            location = item.location
            channel = item.channel
            config_sources[location].append(channel)
        return config_sources


class DialogChannels(DialogBase):
    """Dialog to add delete and select active conda package channels."""

    sig_channels_updated = Signal(object, object)  # added, removed
    sig_setup_ready = Signal()
    sig_check_ready = Signal()
    WIDTH = 550

    def __init__(self, parent=None):
        """Dialog to add delete and select active conda pacakge channels ."""
        super(DialogChannels, self).__init__(parent)
        self._parent = parent
        self._conda_url = 'https://conda.anaconda.org'
        self.api = AnacondaAPI()
        self.initial_sources = None
        self.config_sources = None
        self.style_sheet = None
        self._setup_ready = False
        self._conda_url_setup_ready = False

        # Widgets
        self.list = ListWidgetChannels(parent=self, api=self.api)
        self.label_info = LabelBase('Manage channels you want Navigator to include.')
        self.label_status = LabelBase('Collecting sources...')
        self.progress_bar = QProgressBar(self)
        self.button_add = ButtonNormal('Add...')
        self.button_cancel = ButtonNormal('Cancel')
        self.button_ok = ButtonPrimary('Update channels')

        # Widget setup
        self.frame_title_bar.setVisible(False)
        self.list.setFrameStyle(QFrame.NoFrame)
        self.list.setFrameShape(QFrame.NoFrame)
        self.setWindowFlags(self.windowFlags() | Qt.Popup)
        self.setWindowOpacity(0.96)
        self.setMinimumHeight(300)
        self.setMinimumWidth(self.WIDTH)
        self.setModal(True)

        # Layout
        layout_button = QHBoxLayout()
        layout_button.addWidget(self.label_info)
        layout_button.addStretch()
        layout_button.addWidget(self.button_add)

        layout_ok = QHBoxLayout()
        layout_ok.addWidget(self.label_status)
        layout_ok.addWidget(SpacerHorizontal())
        layout_ok.addWidget(self.progress_bar)
        layout_ok.addWidget(SpacerHorizontal())
        layout_ok.addStretch()
        layout_ok.addWidget(self.button_cancel)
        layout_ok.addWidget(SpacerHorizontal())
        layout_ok.addWidget(self.button_ok)

        layout = QVBoxLayout()
        layout.addLayout(layout_button)
        layout.addWidget(SpacerVertical())
        layout.addWidget(self.list)
        layout.addWidget(SpacerVertical())
        layout.addWidget(SpacerVertical())
        layout.addLayout(layout_ok)
        self.setLayout(layout)

        # Signals
        self.button_add.clicked.connect(self.add_channel)
        self.button_ok.clicked.connect(self.update_channels)
        self.button_cancel.clicked.connect(self.reject)
        self.list.sig_status_updated.connect(self.update_status)
        self.list.sig_channel_added.connect(lambda v=None: self.set_tab_order())
        self.list.sig_channel_added.connect(lambda v=None: self.button_ok.setFocus())
        self.list.sig_channel_removed.connect(lambda v=None: self.set_tab_order())
        self.list.sig_channel_removed.connect(lambda v=None: self.button_ok.setFocus())
        self.list.sig_channel_checked.connect(self.sig_check_ready)
        self.list.sig_channel_status.connect(self.refresh)

        self.button_add.setDisabled(True)
        self.button_ok.setDisabled(True)
        self.button_cancel.setDisabled(False)
        self.update_status(action='Collecting sources...', value=0, max_value=0)

    @staticmethod
    def _group_sources_and_channels(sources):
        """
        Flatten sources and channels dictionary to list of tuples.

        [(source, channel), (source, channel)...]
        """
        grouped = []
        for source, channels in sources.items():
            for channel in channels:
                grouped.append((source, channel))
        return grouped

    def keyPressEvent(self, event):
        """Override Qt method."""
        key = event.key()
        if key in [Qt.Key_Escape]:
            if self.list.is_editing:
                self.refresh()
                self.list.is_editing = False
            else:
                self.reject()

    # --- Public API
    # -------------------------------------------------------------------------
    def update_style_sheet(self, style_sheet=None):
        """Update custom css style sheets."""
        if style_sheet is None:
            self.style_sheet = load_style_sheet()
        else:
            self.style_sheet = style_sheet

        self.setStyleSheet(self.style_sheet)
        self.setMinimumWidth(SASS_VARIABLES.WIDGET_CHANNEL_DIALOG_WIDTH)

        try:
            self.list.update_style_sheet(style_sheet)
        except Exception:
            pass

    def update_api(self, worker, api_info, error):
        """Update api info."""
        channel_alias = api_info.get("channel_alias", {})
        location = channel_alias.get("location", "conda.anaconda.org")
        scheme = channel_alias.get("scheme", "https")
        conda_url = "{}://{}".format(scheme, location)
        self._conda_url = conda_url
        self.list.conda_url = self._conda_url
        self._conda_url_setup_ready = True

        if self._setup_ready:
            self.sig_setup_ready.emit()

    def setup(self, worker, conda_config_data, error):
        """Setup the channels widget."""
        self.config_sources = conda_config_data.get('config_sources')
        self.button_add.setDisabled(False)

        for source, data in self.config_sources.items():
            channels = data.get('channels', [])
            for channel in channels:
                item = ListWidgetItemChannel(channel=channel, location=source)
                item.set_editable(False)
                item.new_channel_help.setVisible(False)
                item.label_location.setVisible(False)
                self.list.addItem(item)

        self.set_tab_order()
        self.button_add.setFocus()
        self.button_ok.setDefault(True)
        self.button_cancel.setEnabled(True)

        self.initial_sources = self.list.sources.copy()
        self.update_status()
        self._setup_ready = True

        if self._conda_url_setup_ready:
            self.sig_setup_ready.emit()

    def set_tab_order(self):
        """Fix the tab ordering in the list."""
        if self.list._items:
            self.setTabOrder(self.button_add, self.list._items[0].button_remove)
            self.setTabOrder(self.list._items[-1].button_remove, self.button_cancel)

        self.setTabOrder(self.button_cancel, self.button_ok)
        self.refresh()

    def add_channel(self):
        """Add new conda channel."""
        user_rc_path = self.api._conda_api.user_rc_path
        item = ListWidgetItemChannel(channel='', location=user_rc_path)
        item.new_channel_help.setVisible(True)
        item.label_location.setVisible(True)
        self.list.addItem(item)
        self.refresh(False)

    def update_channels(self):
        """Update channels list and status."""
        sources = self.list.sources

        original = self._group_sources_and_channels(self.initial_sources)
        updated = self._group_sources_and_channels(sources)

        if sorted(original) != sorted(updated):
            self.sig_channels_updated.emit(*self.sources)
            self.accept()
        else:
            self.reject()

    def refresh(self, channel_status=True):
        """Update enable/disable status based on item count."""
        self.button_add.setEnabled(channel_status and bool(self.list.count))
        self.button_ok.setEnabled(channel_status)
        self.button_cancel.setEnabled(True)

        if self.list.count() == 0:
            self.button_add.setEnabled(True)
            self.button_ok.setEnabled(False)

    def update_status(self, action='', message='', value=None, max_value=None):
        """Update the status and progress bar of the widget."""
        visible = bool(action)
        self.label_status.setText(action)
        self.label_status.setVisible(visible)
        if value is not None and max_value is not None:
            self.progress_bar.setVisible(True)
            self.progress_bar.setRange(0, max_value)
            self.progress_bar.setValue(value)
        else:
            self.progress_bar.setVisible(False)

    @property
    def sources(self):
        """Return sources to add and remove from config."""
        original = self._group_sources_and_channels(self.initial_sources)
        updated = self._group_sources_and_channels(self.list.sources)

        original = set(original)
        updated = set(updated)

        add = updated - original
        remove = original - updated

        return add, remove


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local test."""
    from anaconda_navigator.utils.qthelpers import qapplication
    from anaconda_navigator.utils.styles import load_style_sheet

    app = qapplication()
    api = AnacondaAPI()
    widget = DialogChannels(None)
    widget.update_style_sheet(load_style_sheet())
    widget.show()
    worker = api.conda_config_sources()
    worker_2 = api.api_urls()
    worker.sig_chain_finished.connect(widget.setup)
    worker_2.sig_chain_finished.connect(widget.update_api)
    sys.exit(app.exec_())


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Api client tets."""

# yapf: disable

# Standard library imports
import os

# Third party imports
import pytest
from unittest import mock
from flaky import flaky

from binstar_client import Binstar

# Local imports
from anaconda_navigator.config import CONF
from anaconda_navigator.api import client_api
from anaconda_navigator.api.client_api import ClientAPI, ClientWorker
from anaconda_navigator.api.team_edition_api import TeamEditionAPI


# yapf: enable

NAVIGATOR_TEST_USERNAME = os.environ.get('NAVIGATOR_TEST_USERNAME', )
NAVIGATOR_TEST_PASSWORD = os.environ.get('NAVIGATOR_TEST_PASSWORD')

# Constants used in tests
USER_KEYS = ['created_at', 'location', 'user_type', 'company', 'url', 'name']
DEFAULT_TIMEOUT = 10000

skipif = pytest.mark.skipif
xfail = pytest.mark.xfail


def set_default_configs():
    CONF.set('main', 'logged_api_url', None)
    CONF.set('main', 'anaconda_api_url', 'https://api.anaconda.org')
    CONF.set('main', 'team_edition_api_url', None)
    CONF.set('main', 'enterprise_4_repo_api_url', None)

    os.system('anaconda config --set url https://api.anaconda.org')


@pytest.mark.skip
def test_login(qtbot):
    def _worker_output_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert output is not None
        assert len(output) >= 39
        assert error is None

    api = ClientAPI()
    api.logout()

    worker_login = api.login(
        NAVIGATOR_TEST_USERNAME,
        NAVIGATOR_TEST_PASSWORD,
        'navigator-tests',
        '',
    )
    worker_login.sig_finished.connect(_worker_output_ready)
    with qtbot.waitSignal(worker_login.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass


@pytest.mark.skip
def test_login_bad_password(qtbot):
    def _worker_output_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert output is None
        assert error is not None
        assert 'invalid credentials' in error.lower()
        assert '401' in error

    api = ClientAPI()
    api.set_ssl(True)
    worker = api.login('goanpeca', 'badpass', 'navigator-tests', '')
    worker.sig_finished.connect(_worker_output_ready)

    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass


@pytest.mark.skip
def test_login_bad_username(qtbot):
    def _worker_output_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert output is None
        assert error is not None
        assert 'this user is not enabled to log in' in error.lower()
        assert '401' in error

    api = ClientAPI()
    worker = api.login('adsasdasdxzcasdasda', 'badpass', 'navigator-tests', '')
    worker.sig_finished.connect(_worker_output_ready)

    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass


@skipif(NAVIGATOR_TEST_PASSWORD is None, reason='User and password are available as env variables on CI')
def test_load_licenses(qtbot):
    def _worker_output_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert error is None
        assert isinstance(output, (list))
        if len(output) > 0:
            assert len(output) == 4

    api = ClientAPI()
    api.logout()

    worker = api.login(NAVIGATOR_TEST_USERNAME, NAVIGATOR_TEST_PASSWORD, 'Test', '')
    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass

    worker_licenses = api.get_user_licenses()
    worker_licenses.sig_finished.connect(_worker_output_ready)
    with qtbot.waitSignal(worker_licenses.sig_finished, timeout=10000):
        pass

    # Now check the methods alone not via the worker
    licenses = api._get_user_licenses()
    assert isinstance(licenses, (list))
    if len(licenses) > 0:
        assert len(licenses) == 4


def test_api_url():
    api = ClientAPI()
    beta = 'https://api.beta.anaconda.org'
    default = 'https://api.anaconda.org'

    # Switch from beta and back
    api.set_api_url(beta)
    assert api.get_api_url() == beta
    api.set_api_url(default)
    assert api.get_api_url() == default


@flaky(max_runs=4, min_passes=1)
def test_user_logout(qtbot):
    api = ClientAPI()
    api.logout()

    worker = api.login(NAVIGATOR_TEST_USERNAME, NAVIGATOR_TEST_PASSWORD, 'Test', '')
    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass
    api.logout()

    user = api.user()
    print('Logout user:', user)
    assert isinstance(user, dict)
    assert len(user) == 0


@flaky(max_runs=4, min_passes=1)
@skipif(NAVIGATOR_TEST_PASSWORD is None, reason='User and password are available as env variables on CI')
def test_user_login(qtbot):
    api = ClientAPI()
    api.logout()
    worker_login = api.login(NAVIGATOR_TEST_USERNAME, NAVIGATOR_TEST_PASSWORD, 'navigator-tests', '')
    with qtbot.waitSignal(worker_login.sig_finished, timeout=DEFAULT_TIMEOUT):
        pass
    user = api.user()
    print(user)
    assert isinstance(user, dict)
    assert user.get('login') == NAVIGATOR_TEST_USERNAME
    for key in USER_KEYS:
        assert key in user


@skipif(NAVIGATOR_TEST_PASSWORD is None, reason='User and password are available as env variables on CI')
def test_org_login(qtbot):
    api = ClientAPI()
    api.logout()
    worker_login = api.login(NAVIGATOR_TEST_USERNAME, NAVIGATOR_TEST_PASSWORD, 'navigator-tests', '')
    with qtbot.waitSignal(worker_login.sig_finished, timeout=DEFAULT_TIMEOUT) as blocker:
        blocker
    login = 'goanpeca'
    user = api.organizations(login=login)
    print(user)
    assert isinstance(user, dict)
    assert user.get('login') == login
    for key in USER_KEYS:
        assert key in user


def test_ssl():
    api = ClientAPI()
    default = True
    test = False
    # Switch from beta and back
    api.set_ssl(test)
    assert api.get_ssl() == test
    api.set_ssl(default)
    assert api.get_ssl() == default


def test_client_worker_succeeds(qtbot):
    exception_text = 'Text of exception'

    def method(*args, **kwargs):
        raise Exception('(' + exception_text + ')')

    def worker_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert output is None
        assert exception_text in error

    worker = ClientWorker(method, (1, ), {'test': 2})
    worker.sig_finished.connect(worker_ready)

    assert not worker.is_finished()

    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT) as blocker:
        worker.start()
        blocker

    assert worker.is_finished()


def test_client_worker_fails(qtbot):
    def method(*args, **kwargs):
        return [args, kwargs]

    def worker_ready(worker, output, error):
        print('Output', output)
        print('Error', error)
        assert error is None
        assert [(1, ), {'test': 2}] == output

    worker = ClientWorker(method, (1, ), {'test': 2})
    worker.sig_finished.connect(worker_ready)

    assert not worker.is_finished()

    with qtbot.waitSignal(worker.sig_finished, timeout=DEFAULT_TIMEOUT) as blocker:
        worker.start()
        blocker

    assert worker.is_finished()


def test_reload_api_not_logged_in_user():
    """
    Checks that Binstar client is loaded with the default Anaconda API config.
    """
    CONF.set('main', 'logged_api_url', None)

    api = ClientAPI()
    api.reload_client()
    assert isinstance(api.anaconda_client_api, Binstar)

    set_default_configs()


def test_reload_api_logged_team_edition():
    """
    Checks that TeamEdition client is loaded while Team Edition user is logged in.
    """
    CONF.set('main', 'team_edition_api_url', 'http://some-team-edition-url.com')
    CONF.set('main', 'logged_api_url', 'http://some-team-edition-url.com')

    api = ClientAPI()
    api.reload_client()
    assert isinstance(api.anaconda_client_api, TeamEditionAPI)

    set_default_configs()


def test_reload_api_logged_repo4():
    """
    Checks that Binstar client is loaded while Repo4 user is logged in.
    """
    CONF.set('main', 'enterprise_4_repo_api_url', 'http://some-repo4-url.com')
    CONF.set('main', 'logged_api_url', 'http://some-repo4-url.com')

    api = ClientAPI()
    api.reload_client()
    assert isinstance(api.anaconda_client_api, Binstar)

    set_default_configs()


@mock.patch('binstar_client.Binstar.user', mock.Mock(return_value={'login': 'some_user_name'}))
def test_reload_api_logged_through_anaconda_client_cli():
    """
    Checks that Binstar client is loaded if no login action was done through the Navigator app
    but there was a login action through the CLI by using Anaconda Client.
    """
    api = ClientAPI()
    api.reload_client()
    assert isinstance(api.anaconda_client_api, Binstar)

    set_default_configs()


def test_domain():
    # TODO:
    pass


def test_load_repodata():
    # TODO:
    pass



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""
This folder contains data files bundled with Anaconda Navigator package.

This folder is defined as a python module so that some convenience global
variables can be defined.
"""

# yapf: disable

from __future__ import absolute_import, division, print_function

# Standard library imports
import os.path as osp

# Local imports
from anaconda_navigator.config import CONF_PATH, METADATA_PATH


# yapf: enable

DATA_PATH = osp.dirname(osp.realpath(__file__))
CONTENT_INFO_PATH = osp.join(CONF_PATH, 'videos.json')
LINKS_INFO_PATH = osp.join(DATA_PATH, 'links.json')
CONF_METADATA_PATH = osp.join(METADATA_PATH, 'metadata.json')
BUNDLE_METADATA_COMP_PATH = osp.join(DATA_PATH, 'metadata.json.bz2')
